
/*
 *  Copyright (C) 2018 Juergen Platzer 
 * 
 *  This file is part of ImageResize.
 *  
 *  ImageResize is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  ImageResize is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with ImageResize.  If not, see <http://www.gnu.org/licenses/>.
 */

using System;
using System.IO;
using System.Collections.Generic;
using System.Drawing;
using System.Drawing.Imaging;

public class ImageResize {


  public static void Main(string[] args) {
  
    
    
    //2. try to collect all files that should be moved
    List<string> pFileNames = findAllImageFilesInDirectory(".");
    int iFileNum = pFileNames.Count;
    Console.WriteLine("1. " + iFileNum + " images have to be resized!");
    
    /*for (int i = 0; i < iFileNum; i++) {
      Console.WriteLine(pFileNames[i]);
    }*/
    
    //3. copy the image files and delete them from the camera
    DateTime pNow = DateTime.Now;
    int iErrorCount = 0;
    for (int i = 0; i < iFileNum; i++) {
      
      //3.1. open and save
      try {
        Bitmap pImage = createBitmap(pFileNames[i]);
        string pLogOutput = "   " + (i+1) + " of " + iFileNum + " images ";
        if (pImage != null) {
          pImage.Save(pFileNames[i], ImageFormat.Jpeg);
          pImage.Dispose();
          Console.WriteLine(pLogOutput + "stored.");
        }
        else {
          Console.WriteLine(pLogOutput + "not modified.");
        }
      }
      catch (Exception pException) {
        Console.WriteLine("ERROR: The image " + pFileNames[i] + " could not be saved: " + pException.Message); 
        iErrorCount++;
      }
      
      
    }
    
    Console.WriteLine("");
    Console.WriteLine("");

    Console.WriteLine(iErrorCount + " errors occurred!");
    Console.WriteLine((iFileNum-iErrorCount) + " images were transferred!");
    
    
    
    
    System.Threading.Thread.Sleep(10000);
    return;
    
    
    
    
    
  
  }
  
  
  ///////////////////////////////////////
  // HELPER METHODS                    //
  ///////////////////////////////////////
  private static List<string> findAllImageFilesInDirectory(string pDirectory) {
    //1. make a simple check
    if (pDirectory == null) return null;
    
    //2. find all image files in the current directory
    string[] pImageFileTypes = {".png", ".jpg", ".jpeg", ".gif", ".bmp"};
    int iFileTypeNum = pImageFileTypes.Length;
    
    List<string> pImageFiles = new List<string>();
    for (int i = 0; i < iFileTypeNum; i++) {
      string[] pFiles = Directory.GetFiles(pDirectory, "*" + pImageFileTypes[i], SearchOption.AllDirectories);
      int iFileNum = pFiles.Length;
      for (int j = 0; j < iFileNum; j++) {
        pImageFiles.Add(pFiles[j]);
      }
    }
    
    return pImageFiles;
    
  }
  
  
  private static Bitmap createBitmap(string pFileName) {
    //1. make a simple check 
    if (pFileName == null) return null;
    
    //2. load the image
    Bitmap pImage = new Bitmap(pFileName);
    
    //3. check if the image has to be resized
    int iWidth = pImage.Width;
    int iHeight = pImage.Height;
    
    if ((iWidth > 1024) || (iHeight > 1024)) {
      //3.1. calculate scalefactor
      double dResizeFactor = 1;
      if (iWidth > iHeight) {
        dResizeFactor = 1.0/((double)iWidth/1024.0);
      }
      else {
        dResizeFactor = 1.0/((double)iHeight/1024.0);
      }
      
      //3.2. create scaled image
      int iNewWidth = (int)(iWidth*dResizeFactor);
      int iNewHeight = (int)(iHeight*dResizeFactor);
      Bitmap pScaledImage = new Bitmap(iNewWidth, iNewHeight);
      
      // Scale the bitmap in high quality mode.
      using (Graphics pGraphics = Graphics.FromImage(pScaledImage))
      {
          /*pGraphics.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.HighQuality;
          pGraphics.PixelOffsetMode = System.Drawing.Drawing2D.PixelOffsetMode.HighQuality;
          pGraphics.CompositingQuality = System.Drawing.Drawing2D.CompositingQuality.HighQuality;
          pGraphics.InterpolationMode = System.Drawing.Drawing2D.InterpolationMode.HighQualityBicubic;
          */
         pGraphics.DrawImage(pImage, new Rectangle(0, 0, iNewWidth, iNewHeight), new Rectangle(0, 0, iWidth, iHeight), GraphicsUnit.Pixel);
      }
      
      pImage.Dispose();
      return pScaledImage;
      
    }
    
    
    //4. finally return the image
    pImage.Dispose();
    return null;
  }


}